### -----------------------------------------
### Module:  Process.pm
### Purpose: Launch and control programs
###          in background mode
### Author:  Michael Schilli
### Date:    95/12/28
### -----------------------------------------
package Process;

###------------------------------------------
### Function: Constructor for process object
### Syntax:   $proc_obj=Process->new();
###------------------------------------------
sub new
{ 
  # Bless an anonymous hash to the package
  # for holding the instance variables
  bless {};
}

###------------------------------------------
### Function: Launch a process in background
### Syntax:   $ret = $proc_obj->start("prg");
###------------------------------------------
sub start
{
  my $self  = shift;
  my $func  = shift;

  # Reap zombies
  $SIG{'CHLD'} = sub { wait };

  # Fork 
  # Child:  Execute function specified in bg
  # Parent: Save child pid in instance 
  #         variable and return
  exec "$func" unless $self->{'pid'}=fork();
}

###------------------------------------------
### Function: Check if the process is still 
###           running
### Syntax:   $ret = $proc_obj->poll();
###------------------------------------------
sub poll
{
  my $self      = shift;

  # Check if pid is defined and, if so, 
  # send zero signal to process to check if 
  # it's running
  defined $self->{'pid'} && 
    kill(0, $self->{'pid'});
}


###------------------------------------------
### Function: Terminate a launched process 
###           and 'wait' for it
### Syntax:   $ret = $proc_obj->kill();
###        or $ret = $proc_obj->kill(SIG...);
###------------------------------------------
sub kill
{ 
  my $self = shift;
  my $sig  = shift;

  # If no signal is provided, take SIGTERM 
  # as default
  $sig = SIGTERM unless defined $sig;

  # Function fails if no pid is set
  return undef if !defined $self->{'pid'};

  # Send kill signal
  kill($sig, $self->{'pid'}) || return undef;

  # Reap zombies
  waitpid($self->{'pid'}, 0) == $self->{'pid'} 
    || return undef;

  undef $self->{'pid'};

  1;
}

1;
